from curl2swift.tests.test_request import TEST_REQUEST
from curl2swift.processing.get_response_json import get_response_json


def test_get_response_json():
    json = get_response_json(TEST_REQUEST)
    assert isinstance(json, dict)
    assert json['url'] == 'https://api.github.com/users/defunkt'



import json


def dict_to_pretty_string(the_dict, indent=4):
    if not the_dict:
        return "{}"

    return ("\n" + " " * indent).join(
        json.dumps(the_dict, sort_keys=True, indent=indent, separators=(',', ': ')).splitlines())



import argparse

def get_curl_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('command')
    parser.add_argument('url')
    parser.add_argument('-d', '--data')
    parser.add_argument('-b', '--data-binary', '--data-raw', default=None)
    parser.add_argument('--data-urlencode', '--data-urlencode', action='append', default=[])
    parser.add_argument('-X', default='')
    parser.add_argument('-H', '--header', action='append', default=[])
    parser.add_argument('--compressed', action='store_true')
    parser.add_argument('-k','--insecure', action='store_true')
    parser.add_argument('--user', '-u', default=())
    parser.add_argument('-i','--include', action='store_true')
    parser.add_argument('-s','--silent', action='store_true')
    parser.add_argument('--location', default=None)
    parser.add_argument('--request', default=None)
    parser.add_argument('-v', default=None)

    return parser



from ast import literal_eval
from curl2swift.processing.create_response_model import create_response_model

RESPONSE_JSON = """
{
  "coord": {
    "lon": 14.4208,
    "lat": 50.088
  },
  "weather": [
    {
      "id": 800,
      "main": "Clear",
      "description": "clear sky",
      "icon": "01n"
    }
  ],
  "base": "stations",
  "main": {
    "temp": 13.48,
    "feels_like": 11.15,
    "temp_min": 11.67,
    "temp_max": 15.56,
    "pressure": 1028,
    "humidity": 47
  },
  "visibility": 10000,
  "wind": {
    "speed": 1.03,
    "deg": 120
  },
  "clouds": {
    "all": 0
  },
  "dt": 1617135901,
  "sys": {
    "type": 1,
    "id": 6835,
    "country": "CZ",
    "sunrise": 1617079382,
    "sunset": 1617125412
  },
  "timezone": 7200,
  "id": 3067696,
  "name": "Prague",
  "cod": 200
}
""".strip()

EXPECTED_RESULT = """
    struct Response: Codable {
        let coord : Coord?
        let weather : [Weather]?
        let base : String?
        let main : Main?
        let visibility : Int?
        let wind : Wind?
        let clouds : Clouds?
        let dt : Int?
        let sys : Sys?
        let timezone : Int?
        let id : Int?
        let name : String?
        let cod : Int?

        enum CodingKeys: String, CodingKey {
            case coord = "coord"
            case weather = "weather"
            case base = "base"
            case main = "main"
            case visibility = "visibility"
            case wind = "wind"
            case clouds = "clouds"
            case dt = "dt"
            case sys = "sys"
            case timezone = "timezone"
            case id = "id"
            case name = "name"
            case cod = "cod"
        }
    }

    struct Sys: Codable {
        let type : Int?
        let id : Int?
        let country : String?
        let sunrise : Int?
        let sunset : Int?

        enum CodingKeys: String, CodingKey {
            case type = "type"
            case id = "id"
            case country = "country"
            case sunrise = "sunrise"
            case sunset = "sunset"
        }
    }

    struct Clouds: Codable {
        let all : Int?

        enum CodingKeys: String, CodingKey {
            case all = "all"
        }
    }

    struct Wind: Codable {
        let speed : Double?
        let deg : Int?

        enum CodingKeys: String, CodingKey {
            case speed = "speed"
            case deg = "deg"
        }
    }

    struct Main: Codable {
        let temp : Double?
        let feelsLike : Double?
        let tempMin : Double?
        let tempMax : Double?
        let pressure : Int?
        let humidity : Int?

        enum CodingKeys: String, CodingKey {
            case temp = "temp"
            case feelsLike = "feels_like"
            case tempMin = "temp_min"
            case tempMax = "temp_max"
            case pressure = "pressure"
            case humidity = "humidity"
        }
    }

    struct Weather: Codable {
        let id : Int?
        let main : String?
        let description : String?
        let icon : String?

        enum CodingKeys: String, CodingKey {
            case id = "id"
            case main = "main"
            case description = "description"
            case icon = "icon"
        }
    }

    struct Coord: Codable {
        let lon : Double?
        let lat : Double?

        enum CodingKeys: String, CodingKey {
            case lon = "lon"
            case lat = "lat"
        }
    }
"""

def test_create_response_model():
    json_dict = literal_eval(RESPONSE_JSON)
    response_model = create_response_model(json_dict)
    assert response_model.strip() == EXPECTED_RESULT.strip()



import logging

def setup():
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s [%(levelname)s] %(message)s",
        handlers=[
            logging.FileHandler("debug.log"),
            logging.StreamHandler()
        ]
    )



from curl2swift.tests.test_request import TEST_REQUEST
from curl2swift.parsing.get_parser import get_curl_parser
from curl2swift.processing.create_request import create_request

CURL = """curl -i https://api.github.com/users/defunkt"""


def test_create_request():
    parser = get_curl_parser()
    request = create_request(CURL, parser)
    assert request == TEST_REQUEST.strip()



import os
import shutil
import git

def clone_repo(url, content_type):
    dir_name = url.split('/')[-1].replace('.git', '')
    clone_func = lambda: git.Git(os.getcwd()).clone(url)
    clone_print = lambda: print('Cloning ' + content_type + ' into ' + dir_name)
    if os.path.isdir(os.getcwd() + os.sep + dir_name):
        print('Directory ' + dir_name + ' already exists.')
        response = input('Would you like to delete it and clone again? [y/n]\n')
        if response == 'y':
            print('Deleting ' + dir_name + '.')
            shutil.rmtree(os.getcwd() + os.sep + dir_name)
            clone_print()
            clone_func()
    else:
        clone_print()
        clone_func()



from collections import OrderedDict, namedtuple
from http import cookies
import re
import shlex

ParsedContext = namedtuple('ParsedContext', [
    'method', 'url', 'data', 'data_urlencode', 'headers', 'cookies', 'verify', 'auth'
])


def parse_context(curl_command, parser):
    method = "get"

    tokens = shlex.split(curl_command)
    parsed_args = parser.parse_args(tokens)
    post_data = parsed_args.data or parsed_args.data_binary

    if post_data:
        method = 'post'

    data_urlencoded = parsed_args.data_urlencode

    if parsed_args.X:
        method = parsed_args.X.lower()

    cookie_dict = OrderedDict()
    quoted_headers = OrderedDict()

    for curl_header in parsed_args.header:
        if curl_header.startswith(':'):
            occurrence = [m.start() for m in re.finditer(':', curl_header)]
            header_key, header_value = curl_header[:occurrence[1]], curl_header[occurrence[1] + 1:]
        else:
            header_key, header_value = curl_header.split(":", 1)

        if header_key.lower().strip("$") == 'cookie':
            cookie = cookies.SimpleCookie(header_value)
            for key in cookie:
                cookie_dict[key] = cookie[key].value
        else:
            quoted_headers[header_key] = header_value.strip()

    user = parsed_args.user
    if parsed_args.user:
        user = tuple(user.split(':'))

    return ParsedContext(
        method=method,
        url=parsed_args.url,
        data=post_data,
        headers=quoted_headers,
        cookies=cookie_dict,
        verify=parsed_args.insecure,
        auth=user,
        data_urlencode=data_urlencoded
    )



TEST_TEMPLATE = """
    func test<REQUEST_NAME>Request() {
        let expectation = XCTestExpectation(description: "waiting for reponse")
        <REQUEST_NAME>Request()
            .set(.baseURL("<URL>"))
            .set(.path("<PATH>"))
            <HEADER_SETTERS>
            <BODY_PARAM_SETTERS>
            .makeRxRequest()
            .mapTo(<REQUEST_NAME>Request.Response.self)
            .do(onSuccess: { _ in expectation.fulfill() },
                onError: { _ in XCTFail("The request should succeed") })
            .discardableSubscribe()
        wait(for: [expectation], timeout: 10)
    }
"""



QUERY_PARAM_SETTER = """
    @discardableResult
    func setQueryParam(_ key: QueryParam, _ value: String) -> Self {
        queryParams[key.rawValue] = value
        return self
    }
"""

PATH_PARAM_SETTER = \
    """@discardableResult
    func setPathParameter(_ param: PathParameter, _ value: String) -> Self {
        let placeholder = "{" + param.rawValue + "}"
        path = path.replacingOccurrences(of: placeholder, with: value)
        return self
    }"""

HEADER_PARAM_SETTER = \
    """@discardableResult
    func setHeader(_ key: Header, _ value: String) -> Self {
        headers[key.rawValue] = value
        return self
    }
    """

BODY_PARAM_SETTER = \
    """@discardableResult
    func setBodyParameter(_ key: BodyParameter, _ value: String) -> Self {
        params[key.rawValue] = value
        return self
    }"""

REQUEST_TEMPLATE = """
/// <DESC>
class <REQUEST_NAME>Request: RequestSpecBuilder {

    <RESPONSE>

    enum QueryParam: String {
        <QUERY_PARAMS>
    }

    enum PathParameter: String {
        <PATH_PARAMS>
    }

    enum Header: String {
        <HEADERS>
    }

    enum BodyParameter: String {
        <BODY_PARAMS>
    }

    required init(baseURL: String = "",
                  path: String = "",
                  queryParams: [String: String] = [:],
                  method: HTTPMethod = .get,
                  headers: [String: String] = [:],
                  params: [String: Any] = [:]) {
        super.init(baseURL: baseURL, path: path, method: method, headers: headers, params: params)
        set(.path("<PATH>"))
        <QUERY_PARAMS_INIT>
        set(.method(<METHOD>))
    }
}

extension <REQUEST_NAME>Request {

    <QUERY_PARAM_SETTER>

    <PATH_PARAM_SETTER>

    <HEADER_PARAM_SETTER>

    <BODY_PARAM_SETTER>
}
"""



from curl2swift.parsing.get_parser import get_curl_parser
from curl2swift.parsing.parse_content import parse_curl
from curl2swift.parsing.get_request_name_and_description import get_request_name_and_description

from curl2swift.processing.create_request import create_request
from curl2swift.processing.get_response_json import get_response_json
from curl2swift.processing.create_response_model import create_response_model
from curl2swift.processing.process_test_template import process_test_template
from curl2swift.processing.process_request_template import process_request_template
from curl2swift.processing.prepare_enum_cases import prepare_enum_cases


def run_main_processing():
    parser = get_curl_parser()

    request_name, description = get_request_name_and_description()
    curl, content = parse_curl(parser)
    header_rows = prepare_enum_cases(list(content.headers.keys()), 'header')
    body_param_rows = prepare_enum_cases(content.param_names, 'param')

    request_code = create_request(curl, parser)
    response_json = get_response_json(request_code)
    response_model = create_response_model(response_json)
    process_request_template(
        request_name, description, content, header_rows, body_param_rows, response_model
    )
    process_test_template(request_name, header_rows, body_param_rows, content)



# __main__.py
from curl2swift.handle_options import handle_options
from curl2swift.run_main_processing import run_main_processing
import curl2swift.utils.logger as logger


def main():
    logger.setup()
    handle_options()
    run_main_processing()


if __name__ == "__main__":
    main()



import subprocess
import re

from curl2swift.utils.logger import logging
from curl2swift.templates.test_template import TEST_TEMPLATE
from curl2swift.utils.pprint_color import pprint_color


def process_test_template(request_name, header_rows, body_param_rows, content):
    logging.info('Processing unit test templacte')
    header_setters = []
    for index, header in enumerate(content.headers):
        value = content.headers[header]
        enum_case = re.findall('case (.*) =', header_rows[index])[0]
        header_setters.append('.setHeader(.' + enum_case + ', "' + value + '")')
    body_param_setters = []
    for index, param in enumerate(content.param_names):
        value = param[1]
        enum_case = re.findall('case (.*) =', body_param_rows[index])[0]
        body_param_setters.append('.setBodyParameter(.' + enum_case + ', "' + value + '")')

    processed_template = TEST_TEMPLATE
    if content.headers:
        processed_template = processed_template\
            .replace('<HEADER_SETTERS>', '\n            '.join(header_setters))
    else:
        processed_template = re.sub('.+HEADER_SETTERS>\n', '', processed_template)

    processed_template = processed_template.replace('<URL>', content.url)
    processed_template = processed_template.replace('<PATH>', content.path)

    processed_template = processed_template.replace('<REQUEST_NAME>', request_name)

    if content.param_names:
        processed_template = processed_template\
            .replace('<BODY_PARAM_SETTERS>', '\n            '.join(body_param_setters))
    else:
        processed_template = re.sub('.+<BODY_PARAM_SETTERS>\n', '', processed_template)

    print('\n' + '- ' * 8)
    print('GENERATED TEST:')
    print('' + '- ' * 8 + '\n')
    pprint_color(processed_template)
    print('\n' + '- ' * 12)
    print('END OF GENERATED OUTPUT')
    print('' + '- ' * 12 + '\n')

    should_copy = input('Copy to clipboard? [y/n]\n')

    if should_copy == 'y':
        subprocess.run("pbcopy", universal_newlines=True, input=processed_template)



from curl2swift.utils.logger import logging

def prepare_enum_cases(names, enum_type):
    logging.info('Preparing enum cases for enum type ' + enum_type + ' with names ' + str(names))
    result = []

    for name in names:
        if not name[0]:
            continue
        if enum_type == 'param':
            logging.info('Processing param with name ' + str(name))
            if '_' in name[0]:
                if name[0][0] == '_':
                    name[0] = ''.join(name[0][1:])
                if name[0][-1] == '_':
                    name[0] = ''.join(name[0][:-2])

                split = name[0].split('_')
                split[0] = split[0].lower()
                split[1:] = [word[0].upper() + word[1:] for word in split[1:]]
                processed_name = ''.join(split)
                result.append('case ' + processed_name + ' = ' + '"' + name[0] + '"')
            else:
                the_name = name[0]
                processed_name = the_name[0].lower() + the_name[1:]
                result.append('case ' + processed_name + ' = ' + '"' + the_name + '"')
        elif enum_type == 'header':
            split = name.split('-')
            split[0] = split[0].lower()
            split[1:] = [word[0].upper() + word[1:] for word in split[1:]]
            processed_name = ''.join(split)
            result.append('case ' + processed_name + ' = ' + '"' + name + '"')
    
    logging.info('Created body param rows: ' + str(result))
    return result



from pygments import highlight
from pygments.lexers import SwiftLexer
from pygments.formatters import TerminalFormatter


def pprint_color(obj):
    print(highlight(obj, SwiftLexer(), TerminalFormatter()))



import getopt
import sys
import subprocess
import webbrowser

from curl2swift.utils.clone_repo import clone_repo

def handle_options():
    options, arguments = getopt.getopt(
        sys.argv[1:],
        "ieb",
        ["issue", "example", 'boilerplate', 'curl=']
    )

    for opt, value in options:
        if opt in ["-i", '--issue']:
            webbrowser.open('https://github.com/tomnvt/curl2swift/issues/new')
            sys.exit(0)
        if opt in ["-e", "--example"]:
            clone_repo('https://github.com/tomnvt/curl2swift-example.git', 'example project')
            subprocess.Popen(['open', 'curl2swift-example', '-a', 'Xcode'])
            sys.exit(0)
        if opt in ["-b", "--boilerplate"]:
            clone_repo('https://github.com/tomnvt/curl2swift-boilerplate.git', 'boilerplate code')
            sys.exit(0)
        if opt in ["-c", "--curl"]:
            return {'curl': value}



from ast import literal_eval
from curl2swift.processing.create_response_model import create_response_model

RESPONSE_JSON = """
{
  "Markets": [
    {
      "Code": "ar",
      "Locale": "ar-EG",
      "Name": "Ø§Ù„Ø¹Ø±Ø¨ÙŠØ© (Ù…ØµØ±)",
      "Language": "Ø§Ù„Ø¹Ø±Ø¨ÙŠØ©",
      "Country": "Ù…ØµØ±",
      "Currency": "EGP",
      "SocialNetworks": [],
      "Enabled": False,
      "Configuration": {
          "EverythingOnSale": True
      }
    },
    {
      "Code": "az",
      "Locale": "az-AZ",
      "Name": "AzÉ™rbaycan dili (AzÉ™rbaycan)",
      "Language": "AzÉ™rbaycan dili",
      "Country": "Azerbaijan",
      "Currency": "AZN",
      "SocialNetworks": [],
      "Enabled": True,
      "Object": {
          "EverythingOnSale": False
      }
    }
  ]
}
""".strip()

EXPECTED_RESULT = """
    struct Response: Codable {
        let markets : [Market]?

        enum CodingKeys: String, CodingKey {
            case markets = "Markets"
        }
    }

    struct Market: Codable {
        let code : String?
        let locale : String?
        let name : String?
        let language : String?
        let country : String?
        let currency : String?
        let socialNetworks : [Any]?
        let enabled : Bool?
        let configuration : Configuration?

        enum CodingKeys: String, CodingKey {
            case code = "Code"
            case locale = "Locale"
            case name = "Name"
            case language = "Language"
            case country = "Country"
            case currency = "Currency"
            case socialNetworks = "SocialNetworks"
            case enabled = "Enabled"
            case configuration = "Configuration"
        }
    }

    struct Configuration: Codable {
        let everythingOnSale : Bool?

        enum CodingKeys: String, CodingKey {
            case everythingOnSale = "EverythingOnSale"
        }
    }
"""

def test_create_response_model():
    json_dict = literal_eval(RESPONSE_JSON)
    response_model = create_response_model(json_dict)
    assert response_model.strip() == EXPECTED_RESULT.strip()



from curl2swift.utils.logger import logging
from curl2swift.templates.codable_template import CODABLE_TEMPLATE


submodels = []
DEFAULT_MODEL_NAME = 'Response'


def get_value_type_in_swift(key, value, is_in_list = False):
    if isinstance(value, str):
        return 'String'
    elif isinstance(value, bool):
        return 'Bool'
    elif isinstance(value, int):
        return 'Int'
    elif isinstance(value, float):
        return 'Double'
    elif isinstance(value, list):
        if value:
            return '[' + get_value_type_in_swift(key, value[0], True) + ']'
        return '[Any]'
    elif isinstance(value, dict):
        if is_in_list:
            singular = key[:-1] if key[-1] == 's' else key
            return singular[0].upper() + singular[1:]
        else:
            return key[0].upper() + key[1:]
    else:
        return 'String'



def add_submodel(model_dict, model_name):
    logging.info("Submodel found: " + model_name)
    if model_dict:
        submodels.append(create_response_model(model_dict, model_name))
    else:
        logging.warning("Found empty dictionary in key " + model_name +
                        '. This key will be ignored.')
        return

def create_response_model(response_json, model_name='Response'):
    global submodels
    if model_name == 'Response':
        submodels = []
    model_name = model_name.replace('/', '')
    logging.info("Creating response model for " + model_name)
    properties = []
    coding_keys = []

    for key in response_json:
        value = response_json[key]
        value_type = get_value_type_in_swift(key, value)

        if isinstance(value, dict):
            add_submodel(value, value_type)
        elif isinstance(value, list) and value:
            if isinstance(value[0], dict):
                add_submodel(value[0], value_type[1:-1])

        property_name = key[0].lower() + key[1:]
        property_name = property_name.replace('/', '')
        if "_" in property_name:
            split = property_name.split('_')
            property_name = split[0] + ''.join([word[0].upper() + word[1:] for word in split[1:]])
        properties.append('let ' + property_name + ' : ' + value_type + '?')
        coding_keys.append('case ' + property_name + ' = "' + key + '"')

    processed_response_template = CODABLE_TEMPLATE\
        .replace('<PROPERTIES>', '\n        '.join(properties))

    processed_response_template = processed_response_template\
        .replace('<CODING_KEYS>', '\n            '.join(coding_keys))\
        .replace('<MODEL_NAME>', model_name)
    if model_name == DEFAULT_MODEL_NAME:
        processed_response_template += ''.join(reversed(submodels))
    return processed_response_template



from curl2swift.processing.request_template.process_query_params import process_query_params

def test_process_query_params():
    params = {'q': 'prague', 'appid': 'theAppID', 'units': 'metric'}
    result = process_query_params(params, '<QUERY_PARAMS_INIT>')
    assert result == 'set(.queryParams(["q": "prague", "appid": "theAppID", "units": "metric"]))'


from curl2swift.utils.logger import logging
import re
from curl2swift.templates.request_templates import QUERY_PARAM_SETTER
from curl2swift.constants import TWO_LEVEL_INDENT_SEP


def process_query_params(query_params, processed_template):
    logging.info('Processing query params: ' + str(query_params))
    if query_params:
        processed_template = processed_template.replace('<QUERY_PARAM_SETTER>', QUERY_PARAM_SETTER)
        query_param_key_cases = ['case ' + key for key in list(query_params.keys())]
        processed_template = processed_template\
            .replace('<QUERY_PARAMS>', TWO_LEVEL_INDENT_SEP.join(query_param_key_cases))
        query_param_init_rows = []
        query_params_dict_entries = []
        for query_param_key in query_params:
            query_params_dict_entries.append('"' + query_param_key + '": "' + query_params[query_param_key] + '"')
        query_param_init_row = 'set(.queryParams([' + ', '.join(query_params_dict_entries) + ']))'
        print(query_param_init_row)
        processed_template = processed_template\
            .replace('<QUERY_PARAMS_INIT>', query_param_init_row)
    else:
        processed_template = re\
            .sub(r'\n\s*enum QueryParam: String \{\n.*\n\s*}\n', '', processed_template)
        processed_template = re\
            .sub(r'\n\s*<QUERY_PARAMS>', '', processed_template)
        processed_template = re\
            .sub(r'\n\s*<QUERY_PARAMS_INIT>', '', processed_template)
        processed_template = re\
            .sub(r'\n\s*<QUERY_PARAM_SETTER>', '', processed_template)

    return processed_template




import sys

from curl2swift.utils.logger import logging


def get_request_name_and_description():
    args = sys.argv[1:]
    try:
        request_name = args[0]
    except IndexError:
        logging.warning("Request name missing.")
        request_name = 'Test'

    try:
        description = args[1]
    except IndexError:
        logging.warning('Request description missing.')
        description = 'Add description'
    
    return request_name, description



# pylint: disable=unused-import, exec-used, invalid-name
import requests

_response_json = ''

def get_response_json(request_code):
    exec('global _response_json\n_response_json = (' + request_code + '.json())')
    return _response_json



import os
import sys
import pathlib

from setuptools import setup, find_packages
from setuptools.command.install import install

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# circleci.py version
VERSION = "0.3.0"

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

setup(
    name="curl2swift",
    version=VERSION,
    description="Tool for transforming cURL to Swift code.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/tomnvt/curl2swift",
    author="Tom Novotny",
    author_email="tom.novota@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(include=["curl2swift", "curl2swift.*"]),
    include_package_data=True,
    install_requires=["Pygments", "requests", "pyp3rclip", "gitpython"],
    entry_points={
        "console_scripts": [
            "curl2swift=curl2swift.__main__:main",
        ]
    },    
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)



from curl2swift.processing.request_template.process_query_params import process_query_params
import subprocess
import re

from curl2swift.parsing.parse_content import ParsedContent
from curl2swift.templates.request_templates import REQUEST_TEMPLATE,\
     PATH_PARAM_SETTER, HEADER_PARAM_SETTER, BODY_PARAM_SETTER
from curl2swift.utils.logger import logging
from curl2swift.utils.pprint_color import pprint_color
from curl2swift.constants import TWO_LEVEL_INDENT_SEP


def process_request_template(
    request_name, description, content: ParsedContent, header_rows, body_param_rows, response_model
):
    logging.info('Processing request template')
    processed_template = REQUEST_TEMPLATE
    processed_template = processed_template.replace('<REQUEST_NAME>', request_name)
    processed_template = processed_template.replace('<DESC>', description)
    processed_template = processed_template.replace('<PATH>', content.path)
    processed_template = processed_template.replace('<METHOD>', '.' + content.method.lower())

    processed_template = processed_template\
        .replace('<HEADERS>', TWO_LEVEL_INDENT_SEP.join(header_rows))
    processed_template = processed_template\
        .replace('<BODY_PARAMS>', TWO_LEVEL_INDENT_SEP.join(body_param_rows))

    processed_template = process_query_params(content.query_params, processed_template)

    processed_template = processed_template\
        .replace('<PATH_PARAMS>', TWO_LEVEL_INDENT_SEP.join(content.path_param_rows))
    if content.path_param_rows:
        processed_template = processed_template\
            .replace('<PATH_PARAM_SETTER>', PATH_PARAM_SETTER)
    else:
        processed_template = re\
            .sub(r'\n\s*enum PathParameter: String \{\n\s*\}', '', processed_template)
        processed_template = re\
            .sub(r'\n\s*<PATH_PARAM_SETTER>', '', processed_template)

    if content.headers:
        processed_template = processed_template\
            .replace('<HEADER_PARAM_SETTER>', HEADER_PARAM_SETTER)
    else:
        processed_template = re.sub(r'\n\s*enum Header: String \{\n\s*\}', '', processed_template)
        processed_template = re.sub(r'\n\s*<HEADER_PARAM_SETTER>', '', processed_template)

    if body_param_rows:
        processed_template = processed_template\
            .replace('<BODY_PARAM_SETTER>', BODY_PARAM_SETTER)
    else:
        processed_template = re\
            .sub(r'\n\s*enum BodyParameter: String \{\n\s*\}', '', processed_template)
        processed_template = re.sub(r'\n\s*<BODY_PARAM_SETTER>', '', processed_template)

    processed_template = processed_template.replace('<RESPONSE>', response_model)

    print('\n' + '- ' * 9)
    print('GENERATED REQUEST:')
    print('' + '- ' * 9 + '\n')
    pprint_color(processed_template)
    print('\n' + '- ' * 12)
    print('END OF GENERATED OUTPUT')
    print('' + '- ' * 12 + '\n')

    should_copy = input('Copy output to clipboard? [y/n]\n')

    if should_copy == 'y':
        subprocess.run("pbcopy", universal_newlines=True, input=processed_template)



from curl2swift.utils.dict_to_pretty_string import dict_to_pretty_string
from curl2swift.parsing.parse_context import parse_context


BASE_INDENT = " " * 4


def create_request(curl_command, parser):
    parsed_context = parse_context(curl_command, parser)

    data_token = ''
    if parsed_context.data:
        data_token = '{}data=\'{}\',\n'\
            .format(BASE_INDENT, parsed_context.data)

    if parsed_context.data_urlencode:
        data_token = '{}data=\'{}\',\n'\
            .format(BASE_INDENT, '&'.join(parsed_context.data_urlencode))

    verify_token = ''
    if parsed_context.verify:
        verify_token = '\n{}verify=False'.format(BASE_INDENT)

    auth_data = "{}auth={}".format(BASE_INDENT,parsed_context.auth)

    formatter = {
        'method': parsed_context.method,
        'url': parsed_context.url,
        'data_token': data_token,
        'headers_token': "{}headers={}"\
            .format(BASE_INDENT, dict_to_pretty_string(parsed_context.headers)),
        'cookies_token': "{}cookies={}"\
            .format(BASE_INDENT, dict_to_pretty_string(parsed_context.cookies)),
        'security_token': verify_token,
        'auth': auth_data
    }

    return """requests.{method}("{url}",
{data_token}{headers_token},
{cookies_token},
{auth},{security_token}
)""".format(**formatter)



from collections import namedtuple
from ast import literal_eval
from urllib.parse import urlparse
import sys

import pyp3rclip

from curl2swift.utils.logger import logging
from curl2swift.parsing.parse_context import parse_context

ParsedContent = namedtuple('ParsedContent', ('\
    url,\
    method, \
    path,\
    query_params,\
    headers,\
    param_names,\
    path_param_rows'\
))

def get_curl():
    logging.info('Reading curl from --curl option')
    arguments = ' '.join(sys.argv)
    if '--curl' in sys.argv:
        index = sys.argv.index('--curl')
        curl = sys.argv[index + 1]
        logging.info('Got cURL from option: ' + curl)
    else:
        logging.info('--curl option not used')
        logging.info('Reading curl from clipboard')
        curl = pyp3rclip.paste()

    curl = curl.replace('--location', '')
    curl = curl.replace('-v', '')
    curl = curl.replace('--request', '-X')
    curl = curl.replace('\\\n', '')
    logging.info('cURL after cleanup: ' + curl)
    return curl


def get_parameter_names(context):
    if context.data:
        data = context.data
        if '{' in data:
            data = literal_eval(data)
            return [[param, data[param]] for param in data]
        else:
            return [param.split('=') for param in data.replace(';', '&').split('&')]
    elif context.data_urlencode:
        return [param.split('=') for param in context.data_urlencode]
    return []

def parse_curl(parser):
    curl = get_curl()

    TEST_CURL = "curl -i https://api.github.com/users/defunkt"
    try:
        logging.info('Parsing cURL')
        context = parse_context(curl, parser)
    except:
        logging.error('Parsing failed (see usage above)')
        logging.info('Falling back to test cURL')
        curl = TEST_CURL
        context = parse_context(TEST_CURL, parser)

    logging.info('Transforming cURL to Python request object')
    path_param_rows = [] # NOTE: path params are not yet supported
    param_names = []
    method = context.method
    parsed_url = urlparse(context.url)

    param_names = get_parameter_names(context)
    headers = context.headers
    url = parsed_url.scheme + '://' + parsed_url.netloc
    path = parsed_url.path
    query_params = {param.split('=')[0]: param.split('=')[1]\
        for param in parsed_url.query.split('&')} \
        if parsed_url.query else None

    logging.info('URL: ' + url)
    logging.info('Found method: ' + method)
    logging.info('Found path: ' + path)
    logging.info('Found query params: ' + str(query_params))
    logging.info('Found headers: ' + str(headers))
    logging.info('Found body params: ' + str(param_names))

    content = ParsedContent(url, method, path, query_params, headers, param_names, path_param_rows)
    logging.info("Content parsed.")
    return curl, content
